==> telescope.1.4.zip: source/Telescope/WebViewKeyboardInput.cpp <==
﻿/*
 * Copyright (C) 2020 Activision Publishing, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials
 *    provided with the distribution.
 *
 * 3. Neither the name of Activision Publishing, Inc. nor the names of
 *    its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "WebView.h"
#include "WebFrame.h"
#include "WebPlatformStrategies.h"
#include "WebViewGroup.h"
#include "WebFrameLoaderClient.h"
#include "WebPluginInfoProvider.h"
#include "WebInspectorClient.h"
#include "WebDatabaseProvider.h"
#include "PageStorageSessionProvider.h"
#include "WebEditorClient.h"
#include "WebProgressTrackerClient.h"
#include "BackForwardList.h"
#include "WebFrameNetworkingContext.h"
#include "WebChromeClient.h"
#include "WebVisitedLinkStore.h"
#include "TelescopeSurface.h"

#include <cairo.h>
#include <webcore/Page.h>
#include <webcore/Frame.h>
#include <webcore/NetworkStorageSession.h>
#include <JavaScriptCore/InitializeThreading.h>
#include <WTF/MemoryPressureHandler.h>
#include <WTF/StdUnorderedMap.h>
#include <WebCore/MemoryRelease.h>
#include <webcore/DeprecatedGlobalSettings.h>
#include <WebCore/PageConfiguration.h>
#include <PAL/SessionID.h>
#include <webcore/SocketProvider.h>
#include <webcore/CacheStorageProvider.h>
#include <webcore/CookieJar.h>
#include <webcore/MediaRecorderPrivate.h>
#include <webcore/LibWebRTCProvider.h>
#include <webcore/MediaRecorderProvider.h>
#include <webcore/MIMETypeRegistry.h>
#include <webcore/ApplicationCacheStorage.h>
#include <webcore/RuntimeEnabledFeatures.h>
#include <webcore/URLUtils.h>
#include <webcore/Settings.h>
#include <webcore/DatabaseManager.h>
#include <WebCore/UserContentController.h>
#include <webcore/StorageNamespaceProvider.h>
#include <webcore/PlatformContextCairo.h>
#include <webcore/ProgressTracker.h>
#include <WebCore/FocusController.h>
#include <WebCore/EventHandler.h>
#include <WebCore/PlatformKeyboardEvent.h>
#include <webcore/EventNames.h>
#include <WebCore/Editor.h>
#include <WebCore/KeyboardEvent.h>
#include <fontconfig/fontconfig.h>
#include "PlatformFeatureDefs.h"
#include "TelescopeJavascriptValue.h"
#include "TelescopeAPI.h"
#include "Universal/ViewPrivate.h"

using namespace WebCore;

namespace Telescope
{

static const unsigned CtrlKey = 1 << 0;
static const unsigned AltKey = 1 << 1;
static const unsigned ShiftKey = 1 << 2;

struct KeyDownEntry {
	unsigned virtualKey;
	unsigned modifiers;
	const char* name;
};

struct KeyPressEntry {
	unsigned charCode;
	unsigned modifiers;
	const char* name;
};

#if TELESCOPE_USING(TELESCOPE_PC_PROGRAM)
static const KeyDownEntry keyDownEntries[] = {
	{ VK_LEFT,   0,                  "MoveLeft"                                    },
	{ VK_LEFT,   ShiftKey,           "MoveLeftAndModifySelection"                  },
	{ VK_LEFT,   CtrlKey,            "MoveWordLeft"                                },
	{ VK_LEFT,   CtrlKey | ShiftKey, "MoveWordLeftAndModifySelection"              },
	{ VK_RIGHT,  0,                  "MoveRight"                                   },
	{ VK_RIGHT,  ShiftKey,           "MoveRightAndModifySelection"                 },
	{ VK_RIGHT,  CtrlKey,            "MoveWordRight"                               },
	{ VK_RIGHT,  CtrlKey | ShiftKey, "MoveWordRightAndModifySelection"             },
	{ VK_UP,     0,                  "MoveUp"                                      },
	{ VK_UP,     ShiftKey,           "MoveUpAndModifySelection"                    },
	{ VK_PRIOR,  ShiftKey,           "MovePageUpAndModifySelection"                },
	{ VK_DOWN,   0,                  "MoveDown"                                    },
	{ VK_DOWN,   ShiftKey,           "MoveDownAndModifySelection"                  },
	{ VK_NEXT,   ShiftKey,           "MovePageDownAndModifySelection"              },
	{ VK_PRIOR,  0,                  "MovePageUp"                                  },
	{ VK_NEXT,   0,                  "MovePageDown"                                },
	{ VK_HOME,   0,                  "MoveToBeginningOfLine"                       },
	{ VK_HOME,   ShiftKey,           "MoveToBeginningOfLineAndModifySelection"     },
	{ VK_HOME,   CtrlKey,            "MoveToBeginningOfDocument"                   },
	{ VK_HOME,   CtrlKey | ShiftKey, "MoveToBeginningOfDocumentAndModifySelection" },

	{ VK_END,    0,                  "MoveToEndOfLine"                             },
	{ VK_END,    ShiftKey,           "MoveToEndOfLineAndModifySelection"           },
	{ VK_END,    CtrlKey,            "MoveToEndOfDocument"                         },
	{ VK_END,    CtrlKey | ShiftKey, "MoveToEndOfDocumentAndModifySelection"       },

	{ VK_BACK,   0,                  "DeleteBackward"                              },
	{ VK_BACK,   ShiftKey,           "DeleteBackward"                              },
	{ VK_DELETE, 0,                  "DeleteForward"                               },
	{ VK_BACK,   CtrlKey,            "DeleteWordBackward"                          },
	{ VK_DELETE, CtrlKey,            "DeleteWordForward"                           },

	{ 'B',       CtrlKey,            "ToggleBold"                                  },
	{ 'I',       CtrlKey,            "ToggleItalic"                                },

	{ VK_ESCAPE, 0,                  "Cancel"                                      },
	{ VK_OEM_PERIOD, CtrlKey,        "Cancel"                                      },
	{ VK_TAB,    0,                  "InsertTab"                                   },
	{ VK_TAB,    ShiftKey,           "InsertBacktab"                               },
	{ VK_RETURN, 0,                  "InsertNewline"                               },
	{ VK_RETURN, CtrlKey,            "InsertNewline"                               },
	{ VK_RETURN, AltKey,             "InsertNewline"                               },
	{ VK_RETURN, ShiftKey,           "InsertNewline"                               },
	{ VK_RETURN, AltKey | ShiftKey,  "InsertNewline"                               },

	// It's not quite clear whether clipboard shortcuts and Undo/Redo should be handled
	// in the application or in WebKit. We chose WebKit.
	{ 'C',       CtrlKey,            "Copy"                                        },
	{ 'V',       CtrlKey,            "Paste"                                       },
	{ 'X',       CtrlKey,            "Cut"                                         },
	{ 'A',       CtrlKey,            "SelectAll"                                   },
	{ VK_INSERT, CtrlKey,            "Copy"                                        },
	{ VK_DELETE, ShiftKey,           "Cut"                                         },
	{ VK_INSERT, ShiftKey,           "Paste"                                       },
	{ 'Z',       CtrlKey,            "Undo"                                        },
	{ 'Z',       CtrlKey | ShiftKey, "Redo"                                        },
};
#endif	//	#if TELESCOPE_USING(TELESCOPE_PC_PROGRAM)

static const KeyPressEntry keyPressEntries[] = {
	{ '\t',   0,                  "InsertTab"                                   },
	{ '\t',   ShiftKey,           "InsertBacktab"                               },
	{ '\r',   0,                  "InsertNewline"                               },
	{ '\r',   CtrlKey,            "InsertNewline"                               },
	{ '\r',   AltKey,             "InsertNewline"                               },
	{ '\r',   ShiftKey,           "InsertNewline"                               },
	{ '\r',   AltKey | ShiftKey,  "InsertNewline"                               },
};



const char* WebView::interpretKeyEvent( const KeyboardEvent* evt )
{
	ASSERT( evt->type() == eventNames().keydownEvent || evt->type() == eventNames().keypressEvent );

	static HashMap<int, const char*>* keyDownCommandsMap = 0;
	static HashMap<int, const char*>* keyPressCommandsMap = 0;

	if ( !keyDownCommandsMap ) {
		keyDownCommandsMap = new HashMap<int, const char*>;
		keyPressCommandsMap = new HashMap<int, const char*>;

#if TELESCOPE_USING(TELESCOPE_PC_PROGRAM)
		for ( size_t i = 0; i < WTF_ARRAY_LENGTH( keyDownEntries ); ++i )
			keyDownCommandsMap->set( keyDownEntries[i].modifiers << 16 | keyDownEntries[i].virtualKey, keyDownEntries[i].name );
#endif	//	#if TELESCOPE_USING(TELESCOPE_PC_PROGRAM)

		for ( size_t i = 0; i < WTF_ARRAY_LENGTH( keyPressEntries ); ++i )
			keyPressCommandsMap->set( keyPressEntries[i].modifiers << 16 | keyPressEntries[i].charCode, keyPressEntries[i].name );
	}

	unsigned modifiers = 0;
#if TELESCOPE_USING(TELESCOPE_PC_PROGRAM)
	if ( evt->shiftKey() )
		modifiers |= ShiftKey;
	if ( evt->altKey() )
		modifiers |= AltKey;
	if ( evt->ctrlKey() )
		modifiers |= CtrlKey;
#endif	//	#if TELESCOPE_USING(TELESCOPE_PC_PROGRAM)

	if ( evt->type() == eventNames().keydownEvent ) {
		int mapKey = modifiers << 16 | evt->keyCode();
		return mapKey ? keyDownCommandsMap->get( mapKey ) : 0;
	}

	int mapKey = modifiers << 16 | evt->charCode();

	return mapKey ? keyPressCommandsMap->get( mapKey ) : 0;
}


bool WebView::handleEditingKeyboardEvent( WebCore::KeyboardEvent* event )
{
	auto* frame = downcast<Node>( event->target() )->document().frame();
	ASSERT( frame );

	auto* keyEvent = event->underlyingPlatformEvent();
	if ( !keyEvent || keyEvent->isSystemKey() )  // do not treat this as text input if it's a system key event
		return false;

	auto command = frame->editor().command( interpretKeyEvent( event ) );

	if ( keyEvent->type() == PlatformEvent::RawKeyDown ) {
		// WebKit doesn't have enough information about mode to decide how commands that just insert text if executed via Editor should be treated,
		// so we leave it upon WebCore to either handle them immediately (e.g. Tab that changes focus) or let a keypress event be generated
		// (e.g. Tab that inserts a Tab character, or Enter).
		return !command.isTextInsertion() && command.execute( event );
	}

	if ( command.execute( event ) )
		return true;

	// Don't insert null or control characters as they can result in unexpected behaviour
	if ( event->charCode() < ' ' )
		return false;

	return frame->editor().insertText( keyEvent->text(), event );
}

}

==> telescope.1.15.zip: source/Telescope/WebViewKeyboardInput.cpp <==
﻿/*
 * Copyright (C) 2020 Activision Publishing, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials
 *    provided with the distribution.
 *
 * 3. Neither the name of Activision Publishing, Inc. nor the names of
 *    its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "WebView.h"
#include "WebFrame.h"
#include "WebPlatformStrategies.h"
#include "WebViewGroup.h"
#include "WebFrameLoaderClient.h"
#include "WebPluginInfoProvider.h"
#include "WebInspectorClient.h"
#include "WebDatabaseProvider.h"
#include "PageStorageSessionProvider.h"
#include "WebEditorClient.h"
#include "WebProgressTrackerClient.h"
#include "BackForwardList.h"
#include "WebFrameNetworkingContext.h"
#include "WebChromeClient.h"
#include "WebVisitedLinkStore.h"
#include "TelescopeSurface.h"

#include <cairo.h>
#include <webcore/Page.h>
#include <webcore/Frame.h>
#include <webcore/NetworkStorageSession.h>
#include <JavaScriptCore/InitializeThreading.h>
#include <WTF/MemoryPressureHandler.h>
#include <WTF/StdUnorderedMap.h>
#include <WebCore/MemoryRelease.h>
#include <webcore/DeprecatedGlobalSettings.h>
#include <WebCore/PageConfiguration.h>
#include <PAL/SessionID.h>
#include <webcore/SocketProvider.h>
#include <webcore/CacheStorageProvider.h>
#include <webcore/CookieJar.h>
#include <webcore/MediaRecorderPrivate.h>
#include <webcore/LibWebRTCProvider.h>
#include <webcore/MediaRecorderProvider.h>
#include <webcore/MIMETypeRegistry.h>
#include <webcore/ApplicationCacheStorage.h>
#include <webcore/RuntimeEnabledFeatures.h>
#include <webcore/URLUtils.h>
#include <webcore/Settings.h>
#include <webcore/DatabaseManager.h>
#include <WebCore/UserContentController.h>
#include <webcore/StorageNamespaceProvider.h>
#include <webcore/PlatformContextCairo.h>
#include <webcore/ProgressTracker.h>
#include <WebCore/FocusController.h>
#include <WebCore/EventHandler.h>
#include <WebCore/PlatformKeyboardEvent.h>
#include <webcore/EventNames.h>
#include <WebCore/Editor.h>
#include <WebCore/KeyboardEvent.h>
#include <fontconfig/fontconfig.h>
#include "PlatformFeatureDefs.h"
#include "TelescopeJavascriptValue.h"
#include "TelescopeAPI.h"
#include "Universal/ViewPrivate.h"

using namespace WebCore;

namespace Telescope
{

static const unsigned CtrlKey = 1 << 0;
static const unsigned AltKey = 1 << 1;
static const unsigned ShiftKey = 1 << 2;

struct KeyDownEntry
{
	unsigned virtualKey;
	unsigned modifiers;
	const char *name;
};

struct KeyPressEntry
{
	unsigned charCode;
	unsigned modifiers;
	const char *name;
};

#if TELESCOPE_USING( TELESCOPE_PC_PROGRAM )
static const KeyDownEntry keyDownEntries[] =
{
	{ VK_LEFT, 0, "MoveLeft" },
	{ VK_LEFT, ShiftKey, "MoveLeftAndModifySelection" },
	{ VK_LEFT, CtrlKey, "MoveWordLeft" },
	{ VK_LEFT, CtrlKey | ShiftKey, "MoveWordLeftAndModifySelection" },
	{ VK_RIGHT, 0, "MoveRight" },
	{ VK_RIGHT, ShiftKey, "MoveRightAndModifySelection" },
	{ VK_RIGHT, CtrlKey, "MoveWordRight" },
	{ VK_RIGHT, CtrlKey | ShiftKey, "MoveWordRightAndModifySelection" },
	{ VK_UP, 0, "MoveUp" },
	{ VK_UP, ShiftKey, "MoveUpAndModifySelection" },
	{ VK_PRIOR, ShiftKey, "MovePageUpAndModifySelection" },
	{ VK_DOWN, 0, "MoveDown" },
	{ VK_DOWN, ShiftKey, "MoveDownAndModifySelection" },
	{ VK_NEXT, ShiftKey, "MovePageDownAndModifySelection" },
	{ VK_PRIOR, 0, "MovePageUp" },
	{ VK_NEXT, 0, "MovePageDown" },
	{ VK_HOME, 0, "MoveToBeginningOfLine" },
	{ VK_HOME, ShiftKey, "MoveToBeginningOfLineAndModifySelection" },
	{ VK_HOME, CtrlKey, "MoveToBeginningOfDocument" },
	{ VK_HOME, CtrlKey | ShiftKey, "MoveToBeginningOfDocumentAndModifySelection" },

	{ VK_END, 0, "MoveToEndOfLine" },
	{ VK_END, ShiftKey, "MoveToEndOfLineAndModifySelection" },
	{ VK_END, CtrlKey, "MoveToEndOfDocument" },
	{ VK_END, CtrlKey | ShiftKey, "MoveToEndOfDocumentAndModifySelection" },

	{ VK_BACK, 0, "DeleteBackward" },
	{ VK_BACK, ShiftKey, "DeleteBackward" },
	{ VK_DELETE, 0, "DeleteForward" },
	{ VK_BACK, CtrlKey, "DeleteWordBackward" },
	{ VK_DELETE, CtrlKey, "DeleteWordForward" },

	{ 'B', CtrlKey, "ToggleBold" },
	{ 'I', CtrlKey, "ToggleItalic" },

	{ VK_ESCAPE, 0, "Cancel" },
	{ VK_OEM_PERIOD, CtrlKey, "Cancel" },
	{ VK_TAB, 0, "InsertTab" },
	{ VK_TAB, ShiftKey, "InsertBacktab" },
	{ VK_RETURN, 0, "InsertNewline" },
	{ VK_RETURN, CtrlKey, "InsertNewline" },
	{ VK_RETURN, AltKey, "InsertNewline" },
	{ VK_RETURN, ShiftKey, "InsertNewline" },
	{ VK_RETURN, AltKey | ShiftKey, "InsertNewline" },

	// It's not quite clear whether clipboard shortcuts and Undo/Redo should be handled
	// in the application or in WebKit. We chose WebKit.
	{ 'C', CtrlKey, "Copy" },
	{ 'V', CtrlKey, "Paste" },
	{ 'X', CtrlKey, "Cut" },
	{ 'A', CtrlKey, "SelectAll" },
	{ VK_INSERT, CtrlKey, "Copy" },
	{ VK_DELETE, ShiftKey, "Cut" },
	{ VK_INSERT, ShiftKey, "Paste" },
	{ 'Z', CtrlKey, "Undo" },
	{ 'Z', CtrlKey | ShiftKey, "Redo" },
};
#endif // #if TELESCOPE_USING( TELESCOPE_PC_PROGRAM )

static const KeyPressEntry keyPressEntries[] =
{
	{ '\t', 0, "InsertTab" },
	{ '\t', ShiftKey, "InsertBacktab" },
	{ '\r', 0, "InsertNewline" },
	{ '\r', CtrlKey, "InsertNewline" },
	{ '\r', AltKey, "InsertNewline" },
	{ '\r', ShiftKey, "InsertNewline" },
	{ '\r', AltKey | ShiftKey, "InsertNewline" },
};


const char *WebView::interpretKeyEvent( const KeyboardEvent *evt )
{
	ASSERT( evt->type() == eventNames().keydownEvent || evt->type() == eventNames().keypressEvent );

	static HashMap<int, const char *> *keyDownCommandsMap = 0;
	static HashMap<int, const char *> *keyPressCommandsMap = 0;

	if ( !keyDownCommandsMap )
	{
		keyDownCommandsMap = new HashMap<int, const char *>;
		keyPressCommandsMap = new HashMap<int, const char *>;

#if TELESCOPE_USING( TELESCOPE_PC_PROGRAM )
		for ( size_t i = 0; i < WTF_ARRAY_LENGTH( keyDownEntries ); ++i )
			keyDownCommandsMap->set( keyDownEntries[i].modifiers << 16 | keyDownEntries[i].virtualKey, keyDownEntries[i].name );
#endif // #if TELESCOPE_USING( TELESCOPE_PC_PROGRAM )

		for ( size_t i = 0; i < WTF_ARRAY_LENGTH( keyPressEntries ); ++i )
			keyPressCommandsMap->set( keyPressEntries[i].modifiers << 16 | keyPressEntries[i].charCode, keyPressEntries[i].name );
	}

	unsigned modifiers = 0;
#if TELESCOPE_USING( TELESCOPE_PC_PROGRAM )
	if ( evt->shiftKey() )
	{
		modifiers |= ShiftKey;
	}
	if ( evt->altKey() )
	{
		modifiers |= AltKey;
	}
	if ( evt->ctrlKey() )
	{
		modifiers |= CtrlKey;
	}
#endif // #if TELESCOPE_USING( TELESCOPE_PC_PROGRAM )

	if ( evt->type() == eventNames().keydownEvent )
	{
		int mapKey = modifiers << 16 | evt->keyCode();
		return mapKey ? keyDownCommandsMap->get( mapKey ) : 0;
	}

	int mapKey = modifiers << 16 | evt->charCode();

	return mapKey ? keyPressCommandsMap->get( mapKey ) : 0;
}


bool WebView::handleEditingKeyboardEvent( WebCore::KeyboardEvent *event )
{
	auto *frame = downcast<Node>( event->target() )->document().frame();
	ASSERT( frame );

	auto *keyEvent = event->underlyingPlatformEvent();
	if ( !keyEvent || keyEvent->isSystemKey() ) // do not treat this as text input if it's a system key event
	{
		return false;
	}

	auto command = frame->editor().command( interpretKeyEvent( event ) );

	if ( keyEvent->type() == PlatformEvent::RawKeyDown )
	{
		// WebKit doesn't have enough information about mode to decide how commands that just insert text if executed via Editor should be treated,
		// so we leave it upon WebCore to either handle them immediately (e.g. Tab that changes focus) or let a keypress event be generated
		// (e.g. Tab that inserts a Tab character, or Enter).
		return !command.isTextInsertion() && command.execute( event );
	}

	if ( command.execute( event ) )
	{
		return true;
	}

	// Don't insert null or control characters as they can result in unexpected behaviour
	if ( event->charCode() < ' ' )
	{
		return false;
	}

	return frame->editor().insertText( keyEvent->text(), event );
}

} // namespace Telescope
